/**Complexe.cpp
Ou l'on va définir la classe "Complexe", ainsi que les opérations usuelles (+,*,exp)
**/

#include <cmath>
#include "Complexe.h"


//Constructeur
Complexe::Complexe():m_Re(0),m_Im(0)
{
}

//Constructeur de copie
Complexe::Complexe(const Complexe &k)
{
    m_Re = k.m_Re;
    m_Im = k.m_Im;
}

//Destructeur
Complexe::~Complexe()
{
}

//Retourne la partie Réelle du complexe
float Complexe::getRe() const
{
	return m_Re;
}

//Retourne la partie Im du complexe
float Complexe::getIm() const
{
	return m_Im;
}

//Retourne le module du complexe
float Complexe::getModule() const
{
	return sqrt(m_Im*m_Im + m_Re*m_Re);
}

//Retourne l'argument du complexe
float Complexe::getArg() const
{
	return acos(m_Re / getModule());
}

//Définit la Re du complexe
void Complexe::setRe(float re)
{
	m_Re = re;
}

//Définit la partie Im du complexe
void Complexe::setIm(float im)
{
	m_Im = im;
}

//Définit le complexe par la donnée de son module et son argument ( z = r*e^(i*theta) )
void Complexe::setExp(float module, float arg)
{
	m_Re = module * cos(arg);
	m_Im = module * sin(arg);
}

//Surcharge des opérateurs classiques pour définir les opérations classiques sur les complexes.

//Addition de 2 C
Complexe Complexe::operator+(const Complexe &k)
{
	Complexe resultat;
	resultat.setRe(m_Re + k.getRe());
	resultat.setIm(m_Im + k.getIm());
	return resultat;
}

//Soustraction de 2 C
Complexe Complexe::operator-(const Complexe &k)
{
	Complexe res;
	res.setRe(m_Re - k.getRe());
	res.setIm(m_Im - k.getIm());
	return res;
}

//Affectation d'un complexe
/*
Complexe Complexe::operator=(const Complexe &k)
{
    m_Re = k.getRe();
    m_Im = k.getIm();
    return *this;
}
*/

//Multiplication de 2 C
//z=(a+ib)(a'+ib') = (aa' - bb') + i(ab' + ba')
Complexe Complexe::operator*(const Complexe &k)
{
	Complexe res;
	res.setRe((m_Re * k.getRe()) - (m_Im * k.getIm()));
	res.setIm((m_Re * k.getIm()) + (m_Im * k.getRe()) );
	return res;
}

//Surcharge des opérateurs "sur eux même"
Complexe Complexe::operator+=(const Complexe &k)
{
	m_Re = m_Re + k.getRe();
	m_Im = m_Im + k.getIm();
	return *this;
}

Complexe Complexe::operator-=(const Complexe &k)
{
	m_Re = m_Re - k.getRe();
	m_Im = m_Im - k.getIm();
	return *this;
}

Complexe Complexe::operator*=(const Complexe &k)
{
	m_Re = (m_Re * k.getRe()) - (m_Im * k.getIm());
	m_Im = (m_Re * k.getIm()) + (m_Im * k.getRe());
	return *this;
}

//Opération de RXC dans C

//Addition par un R
Complexe Complexe::operator+(float l)
{
	Complexe res;
	res.setRe(m_Re + l);
	res.setIm(m_Im);
	return res;
}

//Soustraction par un R
Complexe Complexe::operator-(float l)
{
	Complexe res;
	res.setRe(m_Re - l);
	res.setIm(m_Im);
	return res;
}

//Multiplication par un R
Complexe Complexe::operator*(float l)
{
	Complexe res;
	res.setRe(m_Re*l);
	res.setIm(m_Im*l);
	return res;
}


