/****Main.cpp
Programme réalisé dans le cadre du TIPE
Sujet : Transformée de Fourier
Ecrit par Audric SCHILTKNECHT
Programme sous Licence GPL
****/

#include <iostream>
#include <cmath>
#include "Divers.h"
#include "Aleatoire.h"
#include "Complexe.h"
#include "Chronometre.h"
#include "BitReverseal.h"
#include "IO.h"


using namespace std;

int main()
{

    cout << " -- Programme de mesure du temps de calculs d'une FFT -- " << endl;
    cout << " -- Différents algorithmes de calculs vous seront proposés --" << endl;
    cout << endl;

    //On vérifie la fréquence du processeur.
    double Freq = 0;
    if(!FrequenceCpu(&Freq))
    {
        cout << "Fréquence CPU impossible à trouver !" << endl
            << "Le programme ne pourra pas calculer le temps d'execution des algorithmes" << endl;
    }

    cout << "Votre processeur est cadencé à : " << Freq << " Hz." << endl;
    cout << "Une autre mesure sera prise pendant les calculs" << endl;


    unsigned int N = 0;

	cout << "Puissance de 2 souhaitée ?" << endl;
	cout << "Pour la méthode récursive, une puissance inférieur à 20 est souhaitée." << endl;
    cin >> N;

    //On teste si N est trop grand, on risque de ne pas avoir assez de mémoire.
    //Je n'aime pas cette méthode parce qu'elle dépend de l'ordinateur utilisé.
    //Limitation obligée pour procédure récursive
    //N>31 => 2^N ne rentre plus sur un int (2^31 nécessite 2Go de RAM pour les complexes)
    if(N>31)
    {
        while(N>31)
        {
            cout << "N sera trop élevé ! Pas assez de mémoire sur la machine." << endl;
            cout << "Entrez une nouvelle valeur." << endl;
            cin >> N;
        }
    }

    //On définit N, nombre d'élements, comme une puissance de 2.
	N = (unsigned int)pow((double)2,(double)N);


	float *ListeR = new float[N];
    cout << "Vous avez demandé " << N << " valeurs." << endl;

    //On crée la liste qui va stocker les coef obtenus après calculs.
    Complexe *Coef = new Complexe[N];

    //Taille mémoire qui sera utilisée (au max).
    cout << "Mémoire nécessaire : " << 2*N*sizeof(Complexe) << " octets "
         << "(" << 2*N*sizeof(Complexe)/(1024.0*1024.0) << " Mo)." << endl << endl;


    //On affiche le menu du choix de la méthode de calcul.
    int choix = 0;

    cout << "== Choix de la méthode de calcul == " << endl;
    cout << "1 -  Procédure naïve" << endl;
    cout << "2 -  Procédure récursive" << endl;
    cout << "3 -  Procédure itérative" << endl;
    cout << "Autre - Quitter" << endl;

    cin >> choix;
    if (choix != 1 && choix != 2 && choix != 3)
    {
        cout << "Vous avez choisi de quitter ce programme." << endl;
        return 0;
    }

    cout << endl << "== Nombre d'itérations ? ==" << endl;
    int I = 1;
    cin >> I;

/*
    cout << endl << "== Sauvegarde des coefficients obtenus ? ==" << endl
         << "(Seuls ceux obtenus pour la dernière itération seront sauvegardés)" << endl
         << "O  - Oui" << endl
         << "N  - Non" << endl;
    char sauv = 'N';
    cin >> sauv;
*/

    //somme va stocker le temps total mis pour effectuer les calculs.
    double somme = 0;

    for (int i=1; i<=I; i++)
    {
        GeneAleatoire(N,ListeR);//On remplit la liste de nombres générés aléatoirement

        //On appelle la fonction qui va effectuer les calculs, et retourne le temps nécessaire
        double t = Calculs(Coef,ListeR,N,choix);

        //On récupère la fréquence après que le CPU ait "chauffé"
        FrequenceCpu(&Freq);

        //t = Nbre de ticks d'horloge effectué
        //tps(sec) = ticks/frequence (en Hz)
        t = t/Freq;

        //On rajoute t à la somme
        somme += t;

        //Sauvegarde du temps
        //Sauvt(choix, N, t);
    }

    cout << "Les calculs ont demandé : " << somme << " secondes." << endl;
    if(I>1)
    {
        cout    << "En moyenne, on a donc effectué un calcul en : " << somme/I
                << " secondes" << endl;
    }
    Sauvt(choix, N, somme/I);
/*
    //Si on a fait le choix, sauvegarde des données
    if(sauv == 'O')
    {
        SauvC(Coef,N);
        cout << "Sauvegarde effectuée dans le fichier \"Data.txt\"" << endl;
    }
*/

    //On récupère la mémoire allouée.
    delete[] ListeR;
    delete[] Coef;

    //On termine le programme.
	return 0;
}


